
/* Testing for Fundamental Vector Moving Average Reprentations     */
/* co-authored by Bin Chen, Jinho Choi, and Juan Carlos Escanciano */

/* Empirical POWER of MDS Tests */
/* Leeper, Walker and Yang (2013) fiscal foresight model simulation */

/* written by Bin Chen & Jinho Choi */
/* Last updated November 22, 2015    */

/* Note: to save the output files, you need to redirect the current path in the program into yours. */

new;
cls;

DGP = 4;
do until DGP==6; 
/* DGP = 1 for Gaussian error e ~ N(0,1),
   DGP = 2 for non-Gaussian error with t-dist(3), 
   DGP = 3 for non-Gaussian error with Standardized Chi_square dist(3), 
   DGP = 4 for non-Gaussian GARCH(1,1) with t-dist(3),
   DGP = 5 for non-Gaussian GARCH(1,1) with Standardized Chi_square dist(3) */

alter=3;
do until alter==4;




N0= 500;                  @ burn-in obs @

/*** d is the dimension ***/
d = 2;

ITR=1;
iseed1=2011211;
iseed2=1921406;

nu = 30;

ITRmax = 1000;

r90 = ITRmax*0.90; 
r95 = ITRmax*0.95; 
r99 = ITRmax*0.99;

/* U00 and V00 are grid points of (u,v) */
/* We use normal variates for (u,v) */

U00=rndns(d,0.5*nu,iseed1);
u00=u00~-u00;
V00=rndns(d,0.5*nu,iseed2);
v00=v00~-v00;


/* note: Pmax is essentially the total number of lags : 2 to 40 */

/* GAUSSIAN KERNEL N(0,1) */

const2=1/sqrt(2*pi);

proc GK(u);
    local Y;
    Y=const2*exp(-0.5*u.*u);
    retp(Y);
endp;

/* DATA-DEPENDENT CHOICE OF P */

/* KERNEL FUNCTIONS */

Con1=pi;

/* BARTLETT KERNEL */

proc Wbar(u);
    local Y;
    Y=(1-u).*(u.<=1);
    retp(Y);
endp;

/* DANIELL KERNEL */

proc Wdan(u);
    local Y;
    Y=sin(Con1*u)./(Con1*u);
    retp(Y);
endp;

/* PARZEN KERNEL */

proc Wpar(u);
    local X,Y;
    X=(Con1/6)*u;
    Y = (1-6.*X^2+6.*X^3).*(X.<=0.5) + (2.*(1-X)^3).*(X.>0.5).*(X.<=1.0);
    retp(Y);
endp;

/* QS KERNEL */

proc Wqs(u);
    local Y;
    Y=sqrt(5/3)*Con1*u;
    Y=(3./Y^2).*(sin(Y)./Y - cos(Y));
    retp(Y);
endp;


proc Kbar(n0,p0);
    local j,Y,c0;
    c0=cols(p0);
    Y=zeros(N0,c0);
    j=1;
    do until j==N0+1;
    Y[j,.]=Wbar(j./p0);
    j=j+1;
    endo;
    retp(Y);
endp;

proc Kdan(n0,p0);
    local j,Y,c0;
    c0=cols(p0);
    Y=zeros(n0,c0);
    j=1;
    do until j==n0+1;
    Y[j,.]=Wdan(j./p0);
    j=j+1;
    endo;
    retp(Y);
endp;

proc Kpar(n0,p0);
    local j,Y,c0;
    c0=cols(p0);
    Y=zeros(n0,c0);
    j=1;
    do until j==n0+1;
    Y[j,.]=Wpar(j./p0);
    j=j+1;
    endo;
    retp(Y);
endp;

proc Kqs(n0,p0);
    local j,Y,c0;
    c0=cols(p0);
    Y=zeros(n0,c0);
    j=1;
    do until j==n0+1;
    Y[j,.]=Wqs(j./p0);
    j=j+1;
    endo;
    retp(Y);
endp;

/* RANGE OF PRELIMINARY BANDWIDTH Pbar*/

Pmax=31;
Pmin=9; 

/* note: The total number of preliminary lags : 10 to 40 */

/* P0max is the maximum lag order to be integrated numerically. 
It can be set to N-1. But for the kernel with bounded support, 
it can be must less than N. This will speed up the computation */

P0max = 90;

/* JUST USE THE BARTLETT KERNEL FOR AUTOMATIC P */
/* P00 is the preliminary bandwidth vector */

P00=zeros(1,Pmax);
j=1;
do until j==Pmax+1;
P00[1,j]=Pmin+j;
j=j+1;
endo;



/*** LWY (2013) Fiscal Foresight Model Simulation ***/

nlagsmax = 4;

SIC2 = zeros(nlagsmax,ITRmax);
NLAGS_HAT = zeros(1,ITRmax);

/* parameterization */

alpha  = 0.4; 
beta   = 0.99; 
tau_ss = 0.25;

theta = alpha*beta*(1-tau_ss);       

@ as the value of theta, governing the degree of noninvertibility,               @
@ gets closer to zero(one), the problem of invertibility gets more(less) severe. @

kappa = (1-theta)*(tau_ss/(1-tau_ss));



/* simulate capital and tax variables */

do until ITR==ITRmax+1;
if alter==1;
n = 100;                    

elseif alter==2;
n = 250;

elseif alter==3;
N = 500;

endif;


/* nlags = 1; */
 

/* No Fiscal Foresight (q = 0) 
k0_nff   = zeros(2+n0+n+nlags,1);    @ # rows = time -1,0 + burn-ins + nobs @
tau0_nff = zeros(2+n0+n+nlags,1); */

/* Fiscal Foresight (q = 2)    */
k0_ff    = zeros(2+n0+n,1);    @ # rows = time -1,0 + burn-ins + sample size @
tau0_ff  = zeros(2+n0+n,1); 


/* structural shock DGP Processes */

if DGP == 1;
    /* Gaussian error :  e ~ N(0,1) */
    eps = rndn(2+n0+n,2);

elseif DGP == 2;
    /* t-distribution with dof = 3 : e ~ t(3)  */
    eps = cdftci(1-rndu(2+n0+n,2),3);    @ starting from time t-2... when q = 2 @

elseif DGP == 3;
    /* Standardized Chi_square dist(3)  */
    eps = cdfchii(rndu(2+n0+n,2),3+zeros(2+n0+n,2));
    eps = (eps-3)/sqrt(2*3);

elseif DGP == 4;
    /* Conditional heteroskedastic error : Diagonal VEC GARCH(1,1) as in Bollersleve et al. (1988) */
    /* H_t = C + A.*(u_(t-1)*u_(t-1)') + G.*H_t-1   */

    uu  = cdftci(1-rndu(d,2+n0+n),3);    @ t-distribution with dof = 3 : e ~ t(3) @ 
    eps = zeros(2+n0+n,2);
    
    C = eye(d);
    A = 0.3*eye(d);
    G = 0.3*eye(d);
    H = zeros(d,(2+n0+n+1)*d);   
    H[.,1:d] = eye(d);
    
    tt = 1;
    
    do until tt == (2+n0+n)+1;
         hh = chol(H[.,((tt-1)*d+1):tt*d]);
         H[.,(tt*d+1):(tt+1)*d] = C + A.*(hh.*(uu[.,tt]*uu[.,tt]').*hh) + G.*H[.,((tt-1)*d+1):tt*d];
         
         eps[tt,.] = (chol(H[.,((tt-1)*d+1):tt*d])*uu[.,tt])';
           
         tt = tt+1;
    endo;
    
elseif DGP == 5;
    /* Conditional heteroskedastic error : Diagonal VEC GARCH(1,1) as in Bollersleve et al. (1988) */
    /* H_t = C + A.*(u_(t-1)*u_(t-1)') + G.*H_t-1   */
      
    uu  = cdfchii(rndu(2,2+n0+n),3+zeros(2,2+n0+n)); @ standaradized Chi_square dist(3) @
    uu = (uu-3)/sqrt(2*3);

    eps = zeros(2+n0+n,2);
    
    C = eye(d);
    A = 0.3*eye(d);
    G = 0.3*eye(d);
    H = zeros(d,(2+n0+n+1)*d);   
    H[.,1:d] = eye(d);
    
    tt = 1;
    
    do until tt == (2+n0+n)+1;
         hh = chol(H[.,((tt-1)*d+1):tt*d]);
         H[.,(tt*d+1):(tt+1)*d] = C + A.*(hh.*(uu[.,tt]*uu[.,tt]').*hh) + G.*H[.,((tt-1)*d+1):tt*d];
         
         eps[tt,.] = (chol(H[.,((tt-1)*d+1):tt*d])*uu[.,tt])';
           
         tt = tt+1;
    endo;
    
endif;



eps_tau = eps[.,1];
eps_a   = eps[.,2];


nrows = 1;
do until nrows == n0+n+1;

    if nrows == 1;
        /*
        @ NFF model @
        k0_nff[2+1,1]   = eps_a[2+1,1];
        tau0_nff[2+1,1] = eps_tau[2+1,1];
        */
        @ FF model @
        k0_ff[2+1,1]   = eps_a[2+1,1] - kappa*(eps_tau[2+0,1]+theta*eps_tau[2+1,1]);
        tau0_ff[2+1,1] = eps_tau[2+1-2,1];

    else;
        /*
        @ NFF model @
        k0_nff[2+nrows,1]   = alpha*k0_nff[2+nrows-1,1]+eps_a[2+nrows,1];
        tau0_nff[2+nrows,1] = eps_tau[2+nrows,1];
        */
        @ FF model @
        k0_ff[2+nrows,1]   = alpha*k0_ff[2+nrows-1,1]+eps_a[2+nrows,1]-kappa*(eps_tau[2+nrows-1,1]+theta*eps_tau[2+nrows,1]);
        tau0_ff[2+nrows,1] = eps_tau[2+nrows-2,1];
    endif;

    nrows = nrows+1;
endo;
/*
k_nff   =   k0_nff[2+n0+1:2+n0+n+nlags,1];
tau_nff = tau0_nff[2+n0+1:2+n0+n+nlags,1];
*/
k_ff    =   k0_ff[2+n0+1:2+n0+n,1];
tau_ff  = tau0_ff[2+n0+1:2+n0+n,1];


@ VAR estimation @

/* x_nff = k_nff~tau_nff; */            
x_ff  = k_ff~tau_ff;        @ dim = nobs x 2 @

dim = cols(x_ff);

@ VAR estimation using companion form @

e_ff = zeros(n,2*nlagsmax);      @ space for residuals up to nlagsmax @

nlagsh = 1;                       @ choice lag order @
do until nlagsh == nlagsmax+1;
/*
XX_nff = ones(n,dim*nlags+1); 
YY_nff = x_nff[nlags+1:rows(x_nff),.];
*/
XX_ff  = ones(n-nlagsh,dim*nlagsh+1);
YY_ff  = x_ff[nlagsh+1:rows(x_ff),.];

i = 1;
do until i == nlagsh+1;
/* XX_nff[.,dim*(i-1)+1:dim*(i-1)+dim] = x_nff[nlags+1-i:rows(x_nff)-i,.]; */
  XX_ff[.,dim*(i-1)+1:dim*(i-1)+dim] =  x_ff[nlagsh+1-i:rows(x_ff)-i,.];
i = i+1;
endo;
/*
PHI_nff = inv(XX_nff'*XX_nff)*(XX_nff'*YY_nff); @ VAR coefficients            @
ee_nff  = YY_nff-XX_nff*PHI_nff;                @ VAR residuals (nobs x dim)  @
*/
PHI_ff  = inv(XX_ff'*XX_ff)*(XX_ff'*YY_ff);     @ VAR coefficients            @
ee_ff   = YY_ff-XX_ff*PHI_ff;                   @ VAR residuals (nobs x dim)  @


/* X_nff = ee_nff./stdc(ee_nff)'; */
e_ff[1:rows(ee_ff),2*nlagsh-1:2*nlagsh] = ee_ff./stdc(ee_ff)';

sig_ff = (1/rows(ee_ff))*(ee_ff'*ee_ff);

SIC2[nlagsh,ITR] = ln(det(sig_ff)) + ln(rows(ee_ff))/rows(ee_ff)*(d^2*nlagsh);

nlagsh = nlagsh + 1;
endo;

pphat = minindc(SIC2[.,ITR]);
NLAGS_HAT[.,ITR] = pphat;
n = n-pphat;
X = e_ff[1:n,2*pphat-1:2*pphat];     @ Fiscal Foresight model  @             



/* t=j+1 to N */
proc X1(j);
    local Y;
    Y=X[j+1:N,.];
    retp(Y);
endp;

/* t=1 to N-j */
proc X2(j);
    local Y;
    Y=X[1:N-j,.];
    retp(Y);
endp;


proc COVYY(j);
local A,B, Y,Y11,Y12, Y21, Y22;
A=X1(j);
Y11=A[.,1];
Y12=A[.,2];
B=X2(j);
Y21=B[.,1];
Y22=B[.,2];
Y=meanc((Y11).*Y21)-meanc(Y11).*meanc(Y21)+meanc((Y12).*Y22)-meanc(Y12).*meanc(Y22);
retp(Y);
endp;

COVYY11=zeros(1,P0max+1);

j=0;
do until j==P0max+1;

COVYY11[1,j+1]=COVYY(j);

j=j+1;
endo;



/* REAL PART OF MARGINAL ECF OF X(t), ORIGINAL VARIABLE */

proc CX1(v,j);
    retp(cos(X1(j)*v));
endp;

proc CX2(v,j);
    
    retp(cos(X2(j)*v));
endp;

proc SX1(v,j);
        retp(sin(X1(j)*v));
endp;

proc SX2(v,j);
    retp(sin(X2(j)*v));
endp;


/* Below is used to determine lag order Phat from data */ 
/* Real/Imaginary Parts of Sigma(u,v,j) of X(t) */ 

proc Re_xxsgm(u,v,j);
local a1,b1,a2,b2,Z;
a1=CX1(u,j); a1=a1-meanc(a1)'; 
b1=SX1(u,j); b1=b1-meanc(b1)';
a2=CX2(v,j); a2=a2-meanc(a2)';
b2=SX2(v,j); b2=b2-meanc(b2)';
Z=meanc(a1.*a2-b1.*b2);
retp(Z);
endp;

proc Im_xxsgm(u,v,j);
local a1,b1,a2,b2,Z;
a1=CX1(u,j); a1=a1-meanc(a1)'; 
b1=SX1(u,j); b1=b1-meanc(b1)';
a2=CX2(v,j); a2=a2-meanc(a2)';
b2=SX2(v,j); b2=b2-meanc(b2)';
Z=meanc(a1.*b2+b1.*a2);
retp(Z);
endp;



proc Syx10s(v,j);
    local Z0,Z1,Z2,RZ1,IZ1,RZ2,IZ2;
    Z0=x1(j);
    Z1=CX2(v,j);
    Z2=SX2(v,j);
    
    RZ1=meanc(Z0[.,1].*Z1)-meanc(Z0[.,1]).*meanc(Z1);
    IZ1=meanc(Z0[.,1].*Z2)-meanc(Z0[.,1]).*meanc(Z2);
    RZ2=meanc(Z0[.,2].*Z1)-meanc(Z0[.,2]).*meanc(Z1);
    IZ2=meanc(Z0[.,2].*Z2)-meanc(Z0[.,2]).*meanc(Z2);
    retp(RZ1.*RZ1 + IZ1.*IZ1+RZ2.*RZ2 + IZ2.*IZ2);
endp;


/* COMPUTE THE INTEGRALS */
/* WRSXX will be used to select lag orders for Myx(1,0) */ 

WRSXX=zeros(1,P0max+1);


j=0;
do until j==P0max+1;


/* For lag dedetrmination */

/* Compute the integral of real Sigma(u,-u,j).*GK(u) */

WRSXX[1,j+1]=2*meanc(Re_xxsgm(u00,-u00,j));


/* NOTE: I have Checked INTQUAD1(&WIYXsgm, V1) is indeed zero */

j=j+1;
endo;


MS1=zeros(1,P0max);
M1_1=MS1;
M1_2=MS1;



/* the initial integration order: P0max  */



Bar=Kbar(P0max,P00);
Bar2=Bar^2;
j=0;
do until j==P0Max+1;

if j==0;
M1_10 = (1-j/N)*COVYY11[1,j+1]*WRSxx[1,j+1];

else;

MS1[1,j]= (1-j/N)*2*meanc(Syx10s(V00,j));

M1_2[1,j] = j*j*MS1[1,j];
M1_1[1,j] = (1-j/N)*COVYY11[1,j+1]*WRSxx[1,j+1];
endif;

j=j+1;
endo;


MS1=N*MS1;

M1_1=2*M1_1*Bar2+M1_10;
M1_2=2*M1_2*Bar2;

M1=N*2*M1_2./M1_1;

P1_1=1.1447142*M1^(1/3);
P1_2=1.4017333*M1^0.2;
P1_3=1.3934819*M1^0.2;
P1_4=1.4223747*M1^0.2;


lnn=ln(N);

P1_1=(P1_1.>lnn).*P1_1+(P1_1.<=lnn).*lnn;
P1_2=(P1_2.>lnn).*P1_2+(P1_2.<=lnn).*lnn;
P1_3=(P1_3.>lnn).*P1_3+(P1_3.<=lnn).*lnn;
P1_4=(P1_4.>lnn).*P1_4+(P1_4.<=lnn).*lnn;


/* THIS IS 1 by 4*Pmax vector */

P0=P1_1~P1_2~P1_3~P1_4;



/* PART II */
/* COMPUTE TEST STATISTICS */

N1=P0max;

Ka=Kbar(n1,P1_1)~Kdan(n1,P1_2)~Kpar(n1,P1_3)~Kqs(n1,P1_4);


Ka2=Ka.*Ka;
Ka4=Ka2.*Ka2;
Cka=sumc(Ka2);
Cka=Cka';
Dka=sumc(Ka4);
Dka=Dka';


/* mean/variance of M(1,L) */


C10=zeros(P0max,1);
D10=zeros(P0max,P0max);

Pdim=CEIL(maxc(P0'))+1;


proc C10j(v,j);
local Z0,Z1,Z2,Z;

Z0=x1(j)-meanc(x1(j))';
Z1=CX2(v,j); Z1=Z1-meanc(Z1)';
Z2=SX2(v,j); Z2=Z2-meanc(Z2)';
Z=(Z0[.,1].*Z0[.,1]).*(Z1.*Z1+Z2.*Z2)+(Z0[.,2].*Z0[.,2]).*(Z1.*Z1+Z2.*Z2);
Z=meanc(Z);
retp(Z);
endp;


proc D10JL(u,v,j,L);
local RZ11,RZ22,IZ11,IZ22,RZ12,IZ12,Z0,Z2,ZjL,C1,S1,C2,S2;

Z0=x1(j)-meanc(x1(j))';
C1=CX2(u,j); C1=C1-meanc(C1)';
S1=SX2(u,j); S1=S1-meanc(S1)';

ZjL=X[j-L+1:N-L,.];
z2=ZjL*v;
C2=cos(Z2); C2=C2-meanc(C2)';
S2=sin(Z2); S2=S2-meanc(S2)';
RZ11=meanc((Z0[.,1].*Z0[.,1]).*(C1.*C2-S1.*S2));
RZ22=meanc((Z0[.,2].*Z0[.,2]).*(C1.*C2-S1.*S2));
IZ11=meanc((Z0[.,1].*Z0[.,1]).*(C1.*S2+S1.*C2));
IZ22=meanc((Z0[.,2].*Z0[.,2]).*(C1.*S2+S1.*C2));
RZ12=meanc((Z0[.,1].*Z0[.,2]).*(C1.*C2-S1.*S2));
IZ12=meanc((Z0[.,1].*Z0[.,2]).*(C1.*S2+S1.*C2));

retp(RZ11^2+IZ11^2+RZ22^2+IZ22^2+2*(RZ12^2+IZ12^2));
endp;


j=1;
do until j==Pdim+1;

C10[j,1]=2*meanc(C10j(V00,j));


L=1;
do until L==j+1;
/* M(1,0) */

D10_jL=zeros(Nu,1);
q=1;
do until q==Nu+1;
D10_jL[q,1]=meanc(D10jL(u00,V00[.,q],j,L));
q=q+1;
endo;

D10[j,L]=meanc(D10_jL);

D10[L,j]=D10[j,L];

L=L+1;
endo;

j=j+1;
endo;


W10m=Ka2'D10*Ka2;

Wdim=rows(W10m);
W10=zeros(1,Wdim);

j=1;
do until j==Wdim+1;

W10[1,j]=W10m[j,j];

j=j+1;
endo;




/* TEST FOR MARTINGALE DIFFERENCE Myx(1,0) */

MS1=MS1*Ka2;
MS1=(MS1-C10'Ka2)./sqrt(2*W10);

MS=MS1;

if ITR==1;
data0=N~ITR~MS; 
else;
data0=data0|(N~ITR~MS);
endif;
N; ITR;

MS1'; 
ITR=ITR+1;
endo;



/* INDEX for lag Pbar*/

Pbar0=zeros(Pmax,1);
j=1;
do until j==Pmax+1;
Pbar0[j,1]=Pmin+j;
j=j+1;
endo;

j=1;
do until j==1+1;
if j==1;
Ipbar=Pbar0;
Itest=ones(Pmax,1)*j;
else;
Ipbar=Ipbar|Pbar0;
Itest=Itest|ones(Pmax,1)*j;
endif;
j=j+1;
endo;


/* Results based on BARTLETT KERNEL */

j=1;
do until j==1+1;
if j==1;
data1=data0[.,1+1+(j-1)*4*Pmax+0*Pmax+1:1+1+(j-1)*4*Pmax+0*Pmax+Pmax];
else;
data1=data1~data0[.,1+1+(j-1)*4*Pmax+0*Pmax+1:1+1+(j-1)*4*Pmax+0*Pmax+Pmax];
endif;
j=j+1;
endo;

/* sort data to compute ecv */

j=1;
do until j==cols(data1)+1;
data1[.,j]=sortc(data1[.,j],1);
j=j+1;
endo;

Bar_ecv=data1[r90,.]~data1[r95,.]~data1[r99,.];


/* Test statistics values */

/* rejections based on asymptotic critical values */

pvalue1=cdfnc(data1);

BAR_REJ=meanc(pvalue1.<0.10)~meanc(pvalue1.<0.05)~meanc(pvalue1.<0.01);


/* Results based on PARZEN KERNEL */

j=1;
do until j==1+1;
if j==1;
data2=data0[.,1+1+(j-1)*4*Pmax+2*Pmax+1:1+1+(j-1)*4*Pmax+2*Pmax+Pmax];
else;
data2=data2~data0[.,1+1+(j-1)*4*Pmax+2*Pmax+1:1+1+(j-1)*4*Pmax+2*Pmax+Pmax];
endif;
j=j+1;
endo;

/* sort data to compute ecv */

j=1;
do until j==cols(data2)+1;
data2[.,j]=sortc(data2[.,j],1);
j=j+1;
endo;

Par_ecv=data2[r90,.]~data2[r95,.]~data2[r99,.];


/* Statistics Values */

pvalue2=cdfnc(data2);

PAR_REJ=meanc(pvalue2.<0.10)~meanc(pvalue2.<0.05)~meanc(pvalue2.<0.01);


/**the output files***/
outwidth 256;

if ALTER==1; 
output file=d:\LWY_power100_mds_phat.out on;
print "                      ";
print "Fiscal Foresight model";
  if     DGP == 1; print "(DGP6) with Gaussian error e ~ N(0,1)";
  elseif DGP == 2; print "(DGP7) with non-Gaussian error with t-dist(3)"; 
  elseif DGP == 3; print "(DGP8) with non-Gaussian error with Standardized Chi_square dist(3)"; 
  elseif DGP == 4; print "(DGP9) with non-Gaussian GARCH(1,1) with t-dist(3)";
  elseif DGP == 5; print "(DGP10) with non-Gaussian GARCH(1,1) with Standardized Chi_square dist(3)";
  endif;
print "------------------------------";
print "# grid points (nu) = " nu;
print "sample size n = " N;
print "burn-in obs = " n0;
print "number of lags = SIC";
print "max lags = " nlagsmax;

print "alpha = " alpha; 
print "beta  = " beta; 
print "tau_ss = " tau_ss;
print "theta = " theta;
print "------------------------------";

Itest~Ipbar~BAR_REJ~PAR_REJ;
output off;



elseif ALTER==2;
output file=d:\LWY_power250_mds_phat.out on;
print "                      ";
print "Fiscal Foresight model";
  if     DGP == 1; print "(DGP6) with Gaussian error e ~ N(0,1)";
  elseif DGP == 2; print "(DGP7) with non-Gaussian error with t-dist(3)"; 
  elseif DGP == 3; print "(DGP8) with non-Gaussian error with Standardized Chi_square dist(3)"; 
  elseif DGP == 4; print "(DGP9) with non-Gaussian GARCH(1,1) with t-dist(3)";
  elseif DGP == 5; print "(DGP10) with non-Gaussian GARCH(1,1) with Standardized Chi_square dist(3)";
  endif;
print "---------------------------";
print "# grid points (nu) = " nu;
print "sample size n = " N;
print "burn-in obs = " n0;
print "number of lags = SIC";
print "max lags = " nlagsmax;

print "alpha = " alpha; 
print "beta  = " beta; 
print "tau_ss = " tau_ss;
print "theta = " theta;
print "---------------------------";

Itest~Ipbar~BAR_REJ~PAR_REJ;
output off;


elseif ALTER==3;
output file=d:\LWY_power500_mds_phat.out on;
print "                      ";
print "Fiscal Foresight model";
  if     DGP == 1; print "(DGP6) with Gaussian error e ~ N(0,1)";
  elseif DGP == 2; print "(DGP7) with non-Gaussian error with t-dist(3)"; 
  elseif DGP == 3; print "(DGP8) with non-Gaussian error with Standardized Chi_square dist(3)"; 
  elseif DGP == 4; print "(DGP9) with non-Gaussian GARCH(1,1) with t-dist(3)";
  elseif DGP == 5; print "(DGP10) with non-Gaussian GARCH(1,1) with Standardized Chi_square dist(3)";
  endif;
print "---------------------------";
print "# grid points (nu) = " nu;
print "sample size n = " N;
print "burn-in obs = " n0;
print "number of lags = SIC";
print "max lags = " nlagsmax;

print "alpha = " alpha; 
print "beta  = " beta; 
print "tau_ss = " tau_ss;
print "theta = " theta;
print "---------------------------";

Itest~Ipbar~BAR_REJ~PAR_REJ;
output off;

endif;


ALTER=ALTER+1;
endo;


DGP=DGP+1;
endo;


